/****************************************************************************/
/*                         SQL Server Internals v3                          */
/*                           Training Materials                             */
/*                                                                          */
/*                  Written by Dmitri V. Korotkevitch                       */
/*                      http://aboutsqlserver.com                           */
/*                        dk@aboutsqlserver.com                             */
/****************************************************************************/
/*                         Diagnostics Scripts                              */
/*                        Memory and Plan Cache                             */
/****************************************************************************/


-- Cached execution plans. 
select 
	p.usecounts, p.cacheobjtype, p.objtype, p.size_in_bytes,
	t.[text] 
from 
	sys.dm_exec_cached_plans p
		cross apply sys.dm_exec_sql_text(p.plan_handle) t
where 
	p.cacheobjtype like 'Compiled Plan%' 
	-- and usecounts = 1
order by
	--p.objtype desc
	p.usecounts desc
option (recompile)
go

-- Most memory-intensive single-use plans
select top (150)
		[text] as [QueryText], cp.objtype, cp.size_in_bytes
from	sys.dm_exec_cached_plans as cp with (nolock)
		cross apply sys.dm_exec_sql_text(plan_handle)
where	cp.cacheobjtype = N'Compiled Plan'
		and cp.objtype in (N'Adhoc', N'Prepared')
		and cp.usecounts = 1
order by cp.size_in_bytes desc 
option	(recompile);
go


-- What is using Memory
select top 10 
    [type] as [Memory Clerk]
    ,convert(decimal(16,3),sum(pages_kb) / 1024.0) as [Memory Usage(MB)]  
from sys.dm_os_memory_clerks with (nolock)
group by [type]  
order by sum(pages_kb) desc;
go

-- More detailed
dbcc memorystatus;
go


-- Information About Memory Grants 
select 
	mg.session_id
	,t.text as [SQL]
	,qp.query_plan as [Plan]
	,mg.is_small -- SQL Server 2008+
	,mg.dop	
	,mg.query_cost
	,mg.request_time
	,mg.required_memory_kb
	,mg.requested_memory_kb
	,mg.wait_time_ms
	,mg.grant_time
	,mg.granted_memory_kb
	,mg.used_memory_kb
	,mg.max_used_memory_kb
from 
	sys.dm_exec_query_memory_grants mg with (nolock)
		cross apply sys.dm_exec_sql_text(mg.sql_handle) t
		cross apply sys.dm_exec_query_plan(mg.plan_handle) as qp
option (recompile);
go

-- Get some data about memory grants if it is not exposed from sys.dm_exec_query_stats
-- Earlier verions - can get the limited data from the plan. 
;with xmlnamespaces(default 'http://schemas.microsoft.com/sqlserver/2004/07/showplan')
,Statements(PlanHandle, ObjType, UseCount, StmtSimple)
as
(
    select cp.plan_handle, cp.objtype, cp.usecounts, nodes.stmt.query('.')
    from sys.dm_exec_cached_plans cp with (nolock)
        cross apply sys.dm_exec_query_plan(cp.plan_handle) qp
        cross apply qp.query_plan.nodes('//StmtSimple') nodes(stmt)
)
select top 50
    s.PlanHandle, qp.query_plan, s.ObjType, s.UseCount
    ,p.qp.value('@CachedPlanSize','int') as CachedPlanSize
    ,mg.mg.value('@SerialRequiredMemory','int') as [SerialRequiredMemory KB]
    ,mg.mg.value('@SerialDesiredMemory','int') as [SerialDesiredMemory KB]
from Statements s
    cross apply s.StmtSimple.nodes('.//QueryPlan') p(qp)
    cross apply p.qp.nodes('.//MemoryGrantInfo') mg(mg)
	cross apply sys.dm_exec_query_plan(s.PlanHandle) qp
order by 
    mg.mg.value('@SerialRequiredMemory','int') desc;
go


/*** Memory Objects Partitioning and Memory Usage ***/
-- SQL Server 2008+. Do not need in SQL Server 2016
select
	type
	,pages_in_bytes
	, case
		when (creation_options & 0x20 = 0x20) 
			then 'Global PMO. Cannot be partitioned by CPU/NUMA Node. T8048 not applicable.'
		when (creation_options & 0x40 = 0x40) 
			then 'Partitioned by CPU. T8048 not applicable.'
		when (creation_options & 0x80 = 0x80) 
			then 'Partitioned by Node. Use T8048 to further partition by CPU.'
		else
			'Unknown'      
	end as [Partitioning Type]
from 
	sys.dm_os_memory_objects
order by 
	pages_in_bytes desc;
go